/*=========================================================================
| (c) 2015 Total Phase, Inc.
|--------------------------------------------------------------------------
| Project : Promira Sample Code
| File    : capture_espi.py
|--------------------------------------------------------------------------
| Capture eSPI transaction
|--------------------------------------------------------------------------
| Redistribution and use of this file in source and binary forms, with
| or without modification, are permitted.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
 ========================================================================*/

//=========================================================================
// INCLUDES
//=========================================================================
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "promira.h"
#include "promana.h"


//=========================================================================
// CONSTANTS
//=========================================================================


//=========================================================================
// FUNCTIONS (APP)
//=========================================================================
#define APP_NAME            "com.totalphase.promana_espi"

static int dev_open (const char              *ip,
                     Promira                 *pm,
                     PromiraConnectionHandle *conn)
{
    int ret;
    *pm = pm_open(ip);
    if (*pm <= 0) {
        printf("Unable to open Promira platform on %s\n", ip);
        printf("Error code = %d\n", *pm);
        exit(1);
    }
    ret = pm_load(*pm, APP_NAME);
    if (ret < 0) {
        printf("Unable to load the application(%s)\n", APP_NAME);
        printf("Error code = %d\n", ret);
        exit(1);
    }

    *conn = pa_app_connect(ip);
    if (*conn <= 0) {
        printf("Unable to open the application on %s\n", ip);
        printf("Error code = %d\n", *conn);
        exit(1);
    }

    return 0;
}

static void dev_close (Promira                 pm,
                       PromiraConnectionHandle conn)
{
    pa_app_disconnect(conn);
    pm_close(pm);
}

static void print_timestamp (PromiraReadInfo *info)
{
    printf("%4u.%03u.%03u.%03u, %2u.%03u.%03u, ",
           (u32)(info->timestamp / (1000000000L)),
           (u32)(info->timestamp / (1000000L)) % 1000,
           (u32)(info->timestamp / (1000L)) % 1000,
           (u32)(info->timestamp / (1)) % 1000,
           (u32)(info->duration / (1000000000L)) % 1000,
           (u32)(info->duration / (1000000L)) % 1000,
           (u32)(info->duration / (1000L)) % 1000);
}

static void print_status (PromiraReadInfo       *info,
                          PromiraEspiPacketInfo *pkt_info)
{
    // event type
    if (info->events & PA_EVENT_DIGITAL_INPUT)
        printf("Digital Input[%03x]",
               (info->events & PA_EVENT_DIGITAL_INPUT_MASK));
    if (info->events & PA_EVENT_ESPI_ALERT_RISING)
        printf("Alert Rising");
    if (info->events & PA_EVENT_ESPI_ALERT_FALLING)
        printf("Alert Falling");
    if (info->events & PA_EVENT_ESPI_RESET_RISING)
        printf("Reset Rising");
    if (info->events & PA_EVENT_ESPI_RESET_FALLING)
        printf("Reset Falling");
    if (info->events & PA_EVENT_ESPI_INBAND_RESET)
        printf("Inband Reset");
    if (info->events & PA_EVENT_ESPI_PACKET) {
        printf("Packet: ");

        const char *channel_str[] = { "unknown channel", "perif", "vw", "oob",
                                      "flash", "indep" };
        printf("%s ", channel_str[pkt_info->channel + 1]);

        const char *freq_str[] = { "unknown", "20", "25", "33", "50", "66" };
        printf("%s MHz ", freq_str[pkt_info->enum_freq + 1]);

        const char *io_str[] = { "single", "dual", "quad" };
        printf("%s", io_str[pkt_info->io_mode >> 1]);
    }

    // status
    /* error_code = (info->status & PA_READ_ESPI_ERR_CODE_MASK); */
    /* if (error_code) */
    /*     printf(" - Error(%x)", error_code) */
}



//=========================================================================
// STATIC FUNCTIONS
//=========================================================================
static int espi_dump (PromiraConnectionHandle conn, int num_events)
{
    int i = 0, n;

    int ret, slave_id;
    PromiraReadInfo       info;
    PromiraEspiPacketInfo pkt_info;
    u08 data[8192];

    // Start the capture
    if (pa_capture_start(conn, PA_PROTOCOL_ESPI,
                         PA_TRIGGER_MODE_IMMEDIATE) != PA_APP_OK) {
        printf("error: could not enable eSPI capture; exiting...\n");
        exit(1);
    }

    printf("index,slave_id,time(ns),dur(us),status,cmd0 ... cmdN(*),rsp0 ... rspN(*)\n");

    // Capture and print each transaction
    while (i < num_events || num_events == 0) {
        // Read transaction with bit timing data
        ret = pa_espi_read(conn, &info, &pkt_info, sizeof(data), data);

        if (ret == PA_APP_READ_EMPTY)
            continue;

        if (ret < 0)
            break;

        printf("%d,", i);

        slave_id = (info.events & PA_EVENT_SLAVE_ID_MASK) >> 28;
        printf("%d,", slave_id);

        print_timestamp(&info);
        printf("(");
        print_status(&info, &pkt_info);
        printf(")");

        // Check for errors
        i++;
        if (!(info.events & PA_EVENT_ESPI_PACKET)) {
            printf("\n");
            // If zero data captured, continue
            continue;
        }

        printf(", ");

        // Print command
        if (pkt_info.cmd_length > 0) {
            for (n = 0; n < pkt_info.cmd_length - 1; n++) {
                printf("%02x ", data[n]);
            }

            printf("(%02x", data[pkt_info.cmd_length - 1]);
            if (info.status & PA_READ_ESPI_ERR_BAD_CMD_CRC)
                printf("*), ");
            else
                printf("), ");
        }

        // Print response
        if (pkt_info.length - pkt_info.cmd_length > 0) {
            for (n = pkt_info.cmd_length; n < pkt_info.length - 1; n++) {
                printf("%02x ", data[n]);
            }

            printf("(%02x", data[pkt_info.length - 1]);
            if (info.status & PA_READ_ESPI_ERR_BAD_RSP_CRC)
                printf("*)");
            else
                printf(")");
        }

        printf("\n");
    }

    // Stop the capture
    pa_capture_stop(conn);

    return i;
}

//=========================================================================
// USAGE INFORMATION
//=========================================================================
static void print_usage (void)
{
    printf(
"Usage: capture_espi IP num_events\n"
"Example utility for capturing eSPI data from Promira protocol analyzers.\n"
"\n"
"  The parameter num_events is set to the number of events to process\n"
"  before exiting.  If num_events is set to zero, the capture will continue\n"
"  indefinitely.\n"
"\n"
"For product documentation and specifications, see www.totalphase.com.\n");
}


//=========================================================================
// MAIN PROGRAM
//=========================================================================
int main (int argc, char *argv[])
{
    Promira pm;
    PromiraConnectionHandle conn;

    const char * ip;
    int          num_events = 100;
    int          ret;

    if (argc < 3)
    {
        print_usage();
        exit(1);
    }
    ip         = argv[1];
    num_events = atoi(argv[2]);

    // Open the device
    dev_open(ip, &pm, &conn);

    ret = espi_dump(conn, num_events);
    if (ret < 0)
        printf("error: %d\n", ret);

    // Close the device and exit
    dev_close(pm, conn);

    return 0;
}
