/*=========================================================================
| promana Interface Library
|--------------------------------------------------------------------------
| Copyright (c) 2002-2015 Total Phase, Inc.
| All rights reserved.
| www.totalphase.com
|
| Redistribution and use in source and binary forms, with or without
| modification, are permitted provided that the following conditions
| are met:
|
| - Redistributions of source code must retain the above copyright
|   notice, this list of conditions and the following disclaimer.
|
| - Redistributions in binary form must reproduce the above copyright
|   notice, this list of conditions and the following disclaimer in the
|   documentation and/or other materials provided with the distribution.
|
| - Neither the name of Total Phase, Inc. nor the names of its
|   contributors may be used to endorse or promote products derived from
|   this software without specific prior written permission.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
|--------------------------------------------------------------------------
| To access promana through the API:
|
| 1) Use one of the following shared objects:
|      promana.so      --  Linux/Mac OSX shared object
|      promana.dll     --  Windows dynamic link library
|
| 2) Along with one of the following language modules:
|      promana.c/h     --  C/C++ API header file and interface module
|      promana_py.py   --  Python API
|      promana.cs      --  C# .NET source
 ========================================================================*/


/*=========================================================================
| INCLUDES
 ========================================================================*/
/* This #include can be customized to conform to the user's build paths. */
#include "promana.h"


/*=========================================================================
| VERSION CHECK
 ========================================================================*/
#define PA_APP_CFILE_VERSION   0x010a   /* v1.10 */
#define PA_APP_REQ_SW_VERSION  0x010a   /* v1.10 */

/*
 * Make sure that the header file was included and that
 * the version numbers match.
 */
#ifndef PA_APP_HEADER_VERSION
#  error Unable to include header file. Please check include path.

#elif PA_APP_HEADER_VERSION != PA_APP_CFILE_VERSION
#  error Version mismatch between source and header files.

#endif


/*=========================================================================
| DEFINES
 ========================================================================*/
#define API_NAME                     "promana"
#define API_DEBUG                    PA_APP_DEBUG
#define API_OK                       PA_APP_OK
#define API_UNABLE_TO_LOAD_LIBRARY   PA_APP_UNABLE_TO_LOAD_LIBRARY
#define API_INCOMPATIBLE_LIBRARY     PA_APP_INCOMPATIBLE_LIBRARY
#define API_UNABLE_TO_LOAD_FUNCTION  PA_APP_UNABLE_TO_LOAD_FUNCTION
#define API_HEADER_VERSION           PA_APP_HEADER_VERSION
#define API_REQ_SW_VERSION           PA_APP_REQ_SW_VERSION


/*=========================================================================
| LINUX AND DARWIN SUPPORT
 ========================================================================*/
#if defined(__APPLE_CC__) && !defined(DARWIN)
#define DARWIN
#endif

#if defined(linux) || defined(DARWIN)

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#ifdef DARWIN
#define DLOPEN_NO_WARN
#endif

#include <dlfcn.h>

#define DLL_HANDLE  void *
#define MAX_SO_PATH 256

#if API_DEBUG
static char SO_NAME[MAX_SO_PATH+1] = API_NAME ".so";
#endif

#endif


/*=========================================================================
| WINDOWS SUPPORT
 ========================================================================*/
#if defined(WIN32) || defined(_WIN32)

#include <stdio.h>
#include <windows.h>

#define DLL_HANDLE           HINSTANCE
#define dlopen(name, flags)  LoadLibraryA(API_NAME ".dll")
#define dlsym(handle, name)  GetProcAddress(handle, name)
#define dlerror()            "Exiting program"
#if API_DEBUG
#define SO_NAME              API_NAME ".dll"
#endif

#endif


/*=========================================================================
| SHARED LIBRARY LOADER
 ========================================================================*/
/* The error conditions can be customized depending on the application. */
static void *_loadFunction (const char *name, int *result) {
    static DLL_HANDLE handle = 0;
    void * function = 0;

    /* Load the shared library if necessary */
    if (handle == 0) {
        u32 (*version) (void);
        u16 sw_version;
        u16 api_version_req;

        handle = dlopen(0, RTLD_LAZY);
        if (handle == 0) {
#if API_DEBUG
            fprintf(stderr, "Unable to load %s\n", SO_NAME);
            fprintf(stderr, "%s\n", dlerror());
#endif
            *result = API_UNABLE_TO_LOAD_LIBRARY;
            return 0;
        }

        version = (void *)dlsym(handle, "pa_app_c_version");
        if (version == 0) {
#if API_DEBUG
            fprintf(stderr, "Unable to bind pa_app_c_version() in %s\n",
                    SO_NAME);
            fprintf(stderr, "%s\n", dlerror());
#endif
            handle  = 0;
            *result = API_INCOMPATIBLE_LIBRARY;
            return 0;
        }

        sw_version      = (u16)((version() >>  0) & 0xffff);
        api_version_req = (u16)((version() >> 16) & 0xffff);
        if (sw_version  < API_REQ_SW_VERSION ||
            API_HEADER_VERSION < api_version_req)
        {
#if API_DEBUG
            fprintf(stderr, "\nIncompatible versions:\n");

            fprintf(stderr, "  Header version  = v%d.%02d  ",
                    (API_HEADER_VERSION >> 8) & 0xff, API_HEADER_VERSION & 0xff);

            if (sw_version < API_REQ_SW_VERSION)
                fprintf(stderr, "(requires library >= %d.%02d)\n",
                        (API_REQ_SW_VERSION >> 8) & 0xff,
                        API_REQ_SW_VERSION & 0xff);
            else
                fprintf(stderr, "(library version OK)\n");
                        
                   
            fprintf(stderr, "  Library version = v%d.%02d  ",
                    (sw_version >> 8) & 0xff,
                    (sw_version >> 0) & 0xff);

            if (API_HEADER_VERSION < api_version_req)
                fprintf(stderr, "(requires header >= %d.%02d)\n",
                        (api_version_req >> 8) & 0xff,
                        (api_version_req >> 0) & 0xff);
            else
                fprintf(stderr, "(header version OK)\n");
#endif
            handle  = 0;
            *result = API_INCOMPATIBLE_LIBRARY;
            return 0;
        }
    }

    /* Bind the requested function in the shared library */
    function = (void *)dlsym(handle, name);
    *result  = function ? API_OK : API_UNABLE_TO_LOAD_FUNCTION;
    return function;
}


/*=========================================================================
| FUNCTIONS
 ========================================================================*/
static PromiraConnectionHandle (*c_pa_app_connect) (const char *) = 0;
PromiraConnectionHandle pa_app_connect (
    const char * net_addr
)
{
    if (c_pa_app_connect == 0) {
        int res = 0;
        if (!(c_pa_app_connect = _loadFunction("c_pa_app_connect", &res)))
            return res;
    }
    return c_pa_app_connect(net_addr);
}


static int (*c_pa_app_disconnect) (PromiraConnectionHandle) = 0;
int pa_app_disconnect (
    PromiraConnectionHandle conn
)
{
    if (c_pa_app_disconnect == 0) {
        int res = 0;
        if (!(c_pa_app_disconnect = _loadFunction("c_pa_app_disconnect", &res)))
            return res;
    }
    return c_pa_app_disconnect(conn);
}


static int (*c_pa_app_version) (PromiraConnectionHandle, PromiraAppVersion *) = 0;
int pa_app_version (
    PromiraConnectionHandle conn,
    PromiraAppVersion *     version
)
{
    if (c_pa_app_version == 0) {
        int res = 0;
        if (!(c_pa_app_version = _loadFunction("c_pa_app_version", &res)))
            return res;
    }
    return c_pa_app_version(conn, version);
}


static const char * (*c_pa_app_status_string) (int) = 0;
const char * pa_app_status_string (
    int status
)
{
    if (c_pa_app_status_string == 0) {
        int res = 0;
        if (!(c_pa_app_status_string = _loadFunction("c_pa_app_status_string", &res)))
            return 0;
    }
    return c_pa_app_status_string(status);
}


static int (*c_pa_app_features) (PromiraConnectionHandle) = 0;
int pa_app_features (
    PromiraConnectionHandle conn
)
{
    if (c_pa_app_features == 0) {
        int res = 0;
        if (!(c_pa_app_features = _loadFunction("c_pa_app_features", &res)))
            return res;
    }
    return c_pa_app_features(conn);
}


static int (*c_pa_phy_target_power) (PromiraConnectionHandle, u08) = 0;
int pa_phy_target_power (
    PromiraConnectionHandle conn,
    u08                     config
)
{
    if (c_pa_phy_target_power == 0) {
        int res = 0;
        if (!(c_pa_phy_target_power = _loadFunction("c_pa_phy_target_power", &res)))
            return res;
    }
    return c_pa_phy_target_power(conn, config);
}


static f32 (*c_pa_phy_level_shift) (PromiraConnectionHandle, f32) = 0;
f32 pa_phy_level_shift (
    PromiraConnectionHandle conn,
    f32                     level
)
{
    if (c_pa_phy_level_shift == 0) {
        int res = 0;
        if (!(c_pa_phy_level_shift = _loadFunction("c_pa_phy_level_shift", &res)))
            return res;
    }
    return c_pa_phy_level_shift(conn, level);
}


static int (*c_pa_capture_start) (PromiraConnectionHandle, PromiraProtocol, PromiraTriggerMode) = 0;
int pa_capture_start (
    PromiraConnectionHandle conn,
    PromiraProtocol         protocol,
    PromiraTriggerMode      trig_mode
)
{
    if (c_pa_capture_start == 0) {
        int res = 0;
        if (!(c_pa_capture_start = _loadFunction("c_pa_capture_start", &res)))
            return res;
    }
    return c_pa_capture_start(conn, protocol, trig_mode);
}


static int (*c_pa_capture_stop) (PromiraConnectionHandle) = 0;
int pa_capture_stop (
    PromiraConnectionHandle conn
)
{
    if (c_pa_capture_stop == 0) {
        int res = 0;
        if (!(c_pa_capture_stop = _loadFunction("c_pa_capture_stop", &res)))
            return res;
    }
    return c_pa_capture_stop(conn);
}


static int (*c_pa_capture_trigger) (PromiraConnectionHandle) = 0;
int pa_capture_trigger (
    PromiraConnectionHandle conn
)
{
    if (c_pa_capture_trigger == 0) {
        int res = 0;
        if (!(c_pa_capture_trigger = _loadFunction("c_pa_capture_trigger", &res)))
            return res;
    }
    return c_pa_capture_trigger(conn);
}


static int (*c_pa_capture_trigger_wait) (PromiraConnectionHandle, int, PromiraCaptureStatus *) = 0;
int pa_capture_trigger_wait (
    PromiraConnectionHandle conn,
    int                     timeout_ms,
    PromiraCaptureStatus *  status
)
{
    if (c_pa_capture_trigger_wait == 0) {
        int res = 0;
        if (!(c_pa_capture_trigger_wait = _loadFunction("c_pa_capture_trigger_wait", &res)))
            return res;
    }
    return c_pa_capture_trigger_wait(conn, timeout_ms, status);
}


static int (*c_pa_capture_status) (PromiraConnectionHandle, PromiraCaptureStatus *, PromiraCaptureBufferStatus *) = 0;
int pa_capture_status (
    PromiraConnectionHandle      conn,
    PromiraCaptureStatus *       status,
    PromiraCaptureBufferStatus * buf_status
)
{
    if (c_pa_capture_status == 0) {
        int res = 0;
        if (!(c_pa_capture_status = _loadFunction("c_pa_capture_status", &res)))
            return res;
    }
    return c_pa_capture_status(conn, status, buf_status);
}


static int (*c_pa_capture_buffer_config) (PromiraConnectionHandle, u32, u32) = 0;
int pa_capture_buffer_config (
    PromiraConnectionHandle conn,
    u32                     pretrig_kb,
    u32                     capture_kb
)
{
    if (c_pa_capture_buffer_config == 0) {
        int res = 0;
        if (!(c_pa_capture_buffer_config = _loadFunction("c_pa_capture_buffer_config", &res)))
            return res;
    }
    return c_pa_capture_buffer_config(conn, pretrig_kb, capture_kb);
}


static int (*c_pa_capture_buffer_config_query) (PromiraConnectionHandle, u32 *, u32 *) = 0;
int pa_capture_buffer_config_query (
    PromiraConnectionHandle conn,
    u32 *                   pretrig_kb,
    u32 *                   capture_kb
)
{
    if (c_pa_capture_buffer_config_query == 0) {
        int res = 0;
        if (!(c_pa_capture_buffer_config_query = _loadFunction("c_pa_capture_buffer_config_query", &res)))
            return res;
    }
    return c_pa_capture_buffer_config_query(conn, pretrig_kb, capture_kb);
}


static int (*c_pa_digital_io_config) (PromiraConnectionHandle, u32, u32, u32) = 0;
int pa_digital_io_config (
    PromiraConnectionHandle conn,
    u32                     enable,
    u32                     direction,
    u32                     polarity
)
{
    if (c_pa_digital_io_config == 0) {
        int res = 0;
        if (!(c_pa_digital_io_config = _loadFunction("c_pa_digital_io_config", &res)))
            return res;
    }
    return c_pa_digital_io_config(conn, enable, direction, polarity);
}


static int (*c_pa_digital_io_config_query) (PromiraConnectionHandle, u32 *, u32 *, u32 *) = 0;
int pa_digital_io_config_query (
    PromiraConnectionHandle conn,
    u32 *                   enable,
    u32 *                   direction,
    u32 *                   polarity
)
{
    if (c_pa_digital_io_config_query == 0) {
        int res = 0;
        if (!(c_pa_digital_io_config_query = _loadFunction("c_pa_digital_io_config_query", &res)))
            return res;
    }
    return c_pa_digital_io_config_query(conn, enable, direction, polarity);
}


static int (*c_pa_espi_operating_config) (PromiraConnectionHandle, u08, const PromiraEspiOperatingCfg *) = 0;
int pa_espi_operating_config (
    PromiraConnectionHandle         conn,
    u08                             slave_id,
    const PromiraEspiOperatingCfg * cfg
)
{
    if (c_pa_espi_operating_config == 0) {
        int res = 0;
        if (!(c_pa_espi_operating_config = _loadFunction("c_pa_espi_operating_config", &res)))
            return res;
    }
    return c_pa_espi_operating_config(conn, slave_id, cfg);
}


static int (*c_pa_espi_operating_config_read) (PromiraConnectionHandle, u08, PromiraEspiOperatingCfg *) = 0;
int pa_espi_operating_config_read (
    PromiraConnectionHandle   conn,
    u08                       slave_id,
    PromiraEspiOperatingCfg * cfg
)
{
    if (c_pa_espi_operating_config_read == 0) {
        int res = 0;
        if (!(c_pa_espi_operating_config_read = _loadFunction("c_pa_espi_operating_config_read", &res)))
            return res;
    }
    return c_pa_espi_operating_config_read(conn, slave_id, cfg);
}


static int (*c_pa_espi_read) (PromiraConnectionHandle, PromiraReadInfo *, PromiraEspiPacketInfo *, u32, u08 *) = 0;
int pa_espi_read (
    PromiraConnectionHandle conn,
    PromiraReadInfo *       info,
    PromiraEspiPacketInfo * pkt_info,
    u32                     max_bytes,
    u08 *                   packet
)
{
    if (c_pa_espi_read == 0) {
        int res = 0;
        if (!(c_pa_espi_read = _loadFunction("c_pa_espi_read", &res)))
            return res;
    }
    return c_pa_espi_read(conn, info, pkt_info, max_bytes, packet);
}


static int (*c_pa_espi_stats_read) (PromiraConnectionHandle, u08, PromiraEspiStats *) = 0;
int pa_espi_stats_read (
    PromiraConnectionHandle conn,
    u08                     slave_id,
    PromiraEspiStats *      stats
)
{
    if (c_pa_espi_stats_read == 0) {
        int res = 0;
        if (!(c_pa_espi_stats_read = _loadFunction("c_pa_espi_stats_read", &res)))
            return res;
    }
    return c_pa_espi_stats_read(conn, slave_id, stats);
}


static int (*c_pa_espi_hw_filter_config) (PromiraConnectionHandle, u08, const PromiraEspiPacketMatch *) = 0;
int pa_espi_hw_filter_config (
    PromiraConnectionHandle        conn,
    u08                            slave_id,
    const PromiraEspiPacketMatch * pkt_match
)
{
    if (c_pa_espi_hw_filter_config == 0) {
        int res = 0;
        if (!(c_pa_espi_hw_filter_config = _loadFunction("c_pa_espi_hw_filter_config", &res)))
            return res;
    }
    return c_pa_espi_hw_filter_config(conn, slave_id, pkt_match);
}


static int (*c_pa_espi_simple_trigger_config) (PromiraConnectionHandle, u32, const PromiraEspiPacketMatch *, u32) = 0;
int pa_espi_simple_trigger_config (
    PromiraConnectionHandle        conn,
    u32                            act_on,
    const PromiraEspiPacketMatch * pkt_match,
    u32                            actions
)
{
    if (c_pa_espi_simple_trigger_config == 0) {
        int res = 0;
        if (!(c_pa_espi_simple_trigger_config = _loadFunction("c_pa_espi_simple_trigger_config", &res)))
            return res;
    }
    return c_pa_espi_simple_trigger_config(conn, act_on, pkt_match, actions);
}


static int (*c_pa_espi_adv_trig_config) (PromiraConnectionHandle, u08, const PromiraEspiAdvancedTrig1 *, const PromiraEspiAdvancedTrig1 *, const PromiraEspiAdvancedTrig1 *, const PromiraEspiAdvancedTrig1 *, const PromiraEspiAdvancedTrig2 *, const PromiraEspiAdvancedTrigError *) = 0;
int pa_espi_adv_trig_config (
    PromiraConnectionHandle              conn,
    u08                                  slave_id,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level1,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level2,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level3,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level4,
    const PromiraEspiAdvancedTrig2 *     pkt_adv_trig2,
    const PromiraEspiAdvancedTrigError * pkt_adv_trig_error
)
{
    if (c_pa_espi_adv_trig_config == 0) {
        int res = 0;
        if (!(c_pa_espi_adv_trig_config = _loadFunction("c_pa_espi_adv_trig_config", &res)))
            return res;
    }
    return c_pa_espi_adv_trig_config(conn, slave_id, pkt_adv_trig1_level1, pkt_adv_trig1_level2, pkt_adv_trig1_level3, pkt_adv_trig1_level4, pkt_adv_trig2, pkt_adv_trig_error);
}


