/*=========================================================================
| promana Interface Library
|--------------------------------------------------------------------------
| Copyright (c) 2002-2015 Total Phase, Inc.
| All rights reserved.
| www.totalphase.com
|
| Redistribution and use in source and binary forms, with or without
| modification, are permitted provided that the following conditions
| are met:
|
| - Redistributions of source code must retain the above copyright
|   notice, this list of conditions and the following disclaimer.
|
| - Redistributions in binary form must reproduce the above copyright
|   notice, this list of conditions and the following disclaimer in the
|   documentation and/or other materials provided with the distribution.
|
| - Neither the name of Total Phase, Inc. nor the names of its
|   contributors may be used to endorse or promote products derived from
|   this software without specific prior written permission.
|
| THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
| "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
| LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
| FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
| COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
| INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
| BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
| LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
| CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
| LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
| ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
| POSSIBILITY OF SUCH DAMAGE.
|--------------------------------------------------------------------------
| To access promana through the API:
|
| 1) Use one of the following shared objects:
|      promana.so      --  Linux/Mac OSX shared object
|      promana.dll     --  Windows dynamic link library
|
| 2) Along with one of the following language modules:
|      promana.c/h     --  C/C++ API header file and interface module
|      promana_py.py   --  Python API
|      promana.cs      --  C# .NET source
 ========================================================================*/


#ifndef __promana_h__
#define __promana_h__

#ifdef __cplusplus
extern "C" {
#endif


/*=========================================================================
| TYPEDEFS
 ========================================================================*/
#ifndef TOTALPHASE_DATA_TYPES
#define TOTALPHASE_DATA_TYPES

#ifndef _MSC_VER
/* C99-compliant compilers (GCC) */
#include <stdint.h>
typedef uint8_t   u08;
typedef uint16_t  u16;
typedef uint32_t  u32;
typedef uint64_t  u64;
typedef int8_t    s08;
typedef int16_t   s16;
typedef int32_t   s32;
typedef int64_t   s64;

#else
/* Microsoft compilers (Visual C++) */
typedef unsigned __int8   u08;
typedef unsigned __int16  u16;
typedef unsigned __int32  u32;
typedef unsigned __int64  u64;
typedef signed   __int8   s08;
typedef signed   __int16  s16;
typedef signed   __int32  s32;
typedef signed   __int64  s64;

#endif /* __MSC_VER */

typedef float   f32;
typedef double  f64;

#endif /* TOTALPHASE_DATA_TYPES */


/*=========================================================================
| DEBUG
 ========================================================================*/
/* Set the following macro to '1' for debugging */
#define PA_APP_DEBUG 0


/*=========================================================================
| VERSION
 ========================================================================*/
#define PA_APP_HEADER_VERSION  0x010a   /* v1.10 */


/*=========================================================================
| STATUS CODES
 ========================================================================*/
/*
 * All API functions return an integer which is the result of the
 * transaction, or a status code if negative.  The status codes are
 * defined as follows:
 */
enum PromiraAppStatus {
    /* General codes (0 to -99) */
    PA_APP_OK                       =   0,
    PA_APP_UNABLE_TO_LOAD_LIBRARY   =  -1,
    PA_APP_UNABLE_TO_LOAD_DRIVER    =  -2,
    PA_APP_UNABLE_TO_LOAD_FUNCTION  =  -3,
    PA_APP_INCOMPATIBLE_LIBRARY     =  -4,
    PA_APP_INCOMPATIBLE_DEVICE      =  -5,
    PA_APP_COMMUNICATION_ERROR      =  -6,
    PA_APP_UNABLE_TO_OPEN           =  -7,
    PA_APP_UNABLE_TO_CLOSE          =  -8,
    PA_APP_INVALID_HANDLE           =  -9,
    PA_APP_CONFIG_ERROR             = -10,
    PA_APP_MEMORY_ALLOC_ERROR       = -11,
    PA_APP_UNABLE_TO_INIT_SUBSYSTEM = -12,
    PA_APP_INVALID_LICENSE          = -13,
    PA_APP_UNKNOWN_PROTOCOL         = -14,
    PA_APP_STILL_ACTIVE             = -15,
    PA_APP_INACTIVE                 = -16,
    PA_APP_FUNCTION_NOT_AVAILABLE   = -17,
    PA_APP_READ_EMPTY               = -18,

    PA_APP_TIMEOUT                  = -31,
    PA_APP_CONNECTION_LOST          = -32,

    PA_APP_QUEUE_FULL               = -50,

    PA_APP_UNKNOWN_CMD              = -83
};
#ifndef __cplusplus
typedef enum PromiraAppStatus PromiraAppStatus;
#endif


/*=========================================================================
| GENERAL TYPE DEFINITIONS
 ========================================================================*/
/* Connection handle type definition */
typedef int PromiraConnectionHandle;

/*
 * Version matrix.
 *
 * This matrix describes the various version dependencies
 * of Promira components.  It can be used to determine
 * which component caused an incompatibility error.
 *
 * All version numbers are of the format:
 *   (major << 8) | minor
 *
 * ex. v1.20 would be encoded as:  0x0114
 */
struct PromiraAppVersion {
    /* Software, firmware, and hardware versions. */
    u16 software;
    u16 firmware;
    u16 hardware;

    /* Firmware requires that software must be >= this version. */
    u16 sw_req_by_fw;

    /* Software requires that firmware must be >= this version. */
    u16 fw_req_by_sw;

    /* Software requires that the API interface must be >= this version. */
    u16 api_req_by_sw;
};
#ifndef __cplusplus
typedef struct PromiraAppVersion PromiraAppVersion;
#endif


/*=========================================================================
| GENERAL API
 ==========================================================================
| Connect to the application
|
| Returns a connection handle, which is guaranteed to be
| greater than zero if it is vali*/
PromiraConnectionHandle pa_app_connect (
    const char * net_addr
);


/* Disconnect from the application */
int pa_app_disconnect (
    PromiraConnectionHandle conn
);


/*
 * Return the version matrix for the device attached to the
 * given handle.  If the handle is 0 or invalid, only the
 * software and required api versions are set.
 */
int pa_app_version (
    PromiraConnectionHandle conn,
    PromiraAppVersion *     version
);


/*
 * Return the status string for the given status code.
 * If the code is not valid or the library function cannot
 * be loaded, return a NULL string.
 */
const char * pa_app_status_string (
    int status
);


/*
 * Return the device features as a bit-mask of values, or
 * an error code if the handle is not valid.
 */
#define PA_FEATURE_NONE 0x00000000
#define PA_FEATURE_ESPI 0x00000001
int pa_app_features (
    PromiraConnectionHandle conn
);



/*=========================================================================
| POWER API
 ==========================================================================
| Configure the target power pin*/
#define PA_PHY_TARGET_POWER_NONE 0x00
#define PA_PHY_TARGET_POWER_TARGET1_5V 0x01
#define PA_PHY_TARGET_POWER_TARGET1_3V 0x05
#define PA_PHY_TARGET_POWER_TARGET2 0x02
#define PA_PHY_TARGET_POWER_BOTH 0x03
#define PA_PHY_TARGET_POWER_QUERY 0x80
int pa_phy_target_power (
    PromiraConnectionHandle conn,
    u08                     config
);


/* Configure the power of output signal. */
#define PA_PHY_LEVEL_SHIFT_QUERY -1
f32 pa_phy_level_shift (
    PromiraConnectionHandle conn,
    f32                     level
);



/*=========================================================================
| CAPTURE API
 ==========================================================================
| Protocol cod*/
enum PromiraProtocol {
    PA_PROTOCOL_NONE = 0,
    PA_PROTOCOL_ESPI = 1
};
#ifndef __cplusplus
typedef enum PromiraProtocol PromiraProtocol;
#endif

/* Trigger modes */
enum PromiraTriggerMode {
    PA_TRIGGER_MODE_EVENT     = 0,
    PA_TRIGGER_MODE_IMMEDIATE = 1
};
#ifndef __cplusplus
typedef enum PromiraTriggerMode PromiraTriggerMode;
#endif

/* Start capturing */
int pa_capture_start (
    PromiraConnectionHandle conn,
    PromiraProtocol         protocol,
    PromiraTriggerMode      trig_mode
);


/* Stop capturing data */
int pa_capture_stop (
    PromiraConnectionHandle conn
);


/* Trigger capture manually */
int pa_capture_trigger (
    PromiraConnectionHandle conn
);


/*
 * Capture status; general across protocols but used in
 * protocol-specific capture-status-query functions as well
 */
enum PromiraCaptureStatus {
    PA_CAPTURE_STATUS_UNKNOWN          = -1,
    PA_CAPTURE_STATUS_INACTIVE         =  0,
    PA_CAPTURE_STATUS_PRE_TRIGGER      =  1,
    PA_CAPTURE_STATUS_PRE_TRIGGER_SYNC =  2,
    PA_CAPTURE_STATUS_POST_TRIGGER     =  3,
    PA_CAPTURE_STATUS_TRANSFER         =  4,
    PA_CAPTURE_STATUS_COMPLETE         =  5
};
#ifndef __cplusplus
typedef enum PromiraCaptureStatus PromiraCaptureStatus;
#endif

/* Wait for capture to trigger */
int pa_capture_trigger_wait (
    PromiraConnectionHandle conn,
    int                     timeout_ms,
    PromiraCaptureStatus *  status
);


struct PromiraCaptureBufferStatus {
    u32 pretrig_remaining_kb;
    u32 pretrig_total_kb;
    u32 capture_remaining_kb;
    u32 capture_total_kb;
};
#ifndef __cplusplus
typedef struct PromiraCaptureBufferStatus PromiraCaptureBufferStatus;
#endif

int pa_capture_status (
    PromiraConnectionHandle      conn,
    PromiraCaptureStatus *       status,
    PromiraCaptureBufferStatus * buf_status
);


/* General constants */
int pa_capture_buffer_config (
    PromiraConnectionHandle conn,
    u32                     pretrig_kb,
    u32                     capture_kb
);


int pa_capture_buffer_config_query (
    PromiraConnectionHandle conn,
    u32 *                   pretrig_kb,
    u32 *                   capture_kb
);



/*=========================================================================
| DIGITAL INPUT/OUTPUT CONFIGURATION
 ========================================================================*/
#define PA_DIGITAL_DIR_INPUT 0
#define PA_DIGITAL_DIR_OUTPUT 1
#define PA_DIGITAL_ACTIVE_LOW 0
#define PA_DIGITAL_ACTIVE_HIGH 1
/* Configure the digital I/Os */
int pa_digital_io_config (
    PromiraConnectionHandle conn,
    u32                     enable,
    u32                     direction,
    u32                     polarity
);


int pa_digital_io_config_query (
    PromiraConnectionHandle conn,
    u32 *                   enable,
    u32 *                   direction,
    u32 *                   polarity
);



/*=========================================================================
| COMMON CAPTURE TRIGGER OPTIONS
 ==========================================================================
| Packet matching mod*/
enum PromiraMatchType {
    PA_MATCH_TYPE_DISABLED  = 0,
    PA_MATCH_TYPE_EQUAL     = 1,
    PA_MATCH_TYPE_NOT_EQUAL = 2
};
#ifndef __cplusplus
typedef enum PromiraMatchType PromiraMatchType;
#endif


/*=========================================================================
| COMMON STATUS/EVENT FOR READ
 ========================================================================*/
struct PromiraReadInfo {
    u64 timestamp;
    u64 duration;
    u32 status;
    u32 events;
};
#ifndef __cplusplus
typedef struct PromiraReadInfo PromiraReadInfo;
#endif

/* Status */
#define PA_READ_OK 0
#define PA_READ_ERR_CODE_MASK 0x000000ff
/* Event */
#define PA_EVENT_DIGITAL_INPUT_MASK 0x00000fff
#define PA_EVENT_DIGITAL_INPUT 0x00001000
#define PA_EVENT_SLAVE_ID_MASK 0x30000000
#define PA_EVENT_SLAVE0 0x00000000
#define PA_EVENT_SLAVE1 0x10000000
#define PA_EVENT_SLAVE2 0x20000000
#define PA_EVENT_SLAVE3 0x30000000
#define PA_EVENT_TRIGGER 0x80000000
/* SPI IO Mode for SPI and eSPI */
enum PromiraSpiIOMode {
    PA_SPI_IO_UNKNOWN  = -1,
    PA_SPI_IO_STANDARD =  0,
    PA_SPI_IO_DUAL     =  2,
    PA_SPI_IO_QUAD     =  4
};
#ifndef __cplusplus
typedef enum PromiraSpiIOMode PromiraSpiIOMode;
#endif


/*=========================================================================
| eSPI API
 ========================================================================*/
enum PromiraEspiAlertPin {
    PA_ESPI_ALERT_UNKNOWN = 0,
    PA_ESPI_ALERT_PIN     = 1,
    PA_ESPI_ALERT_IO1     = 2
};
#ifndef __cplusplus
typedef enum PromiraEspiAlertPin PromiraEspiAlertPin;
#endif

enum PromiraEspiAlign {
    PA_ESPI_ALIGN_UNKNOWN    = 0,
    PA_ESPI_ALIGN_64_BYTES   = 1,
    PA_ESPI_ALIGN_128_BYTES  = 2,
    PA_ESPI_ALIGN_256_BYTES  = 3,
    PA_ESPI_ALIGN_512_BYTES  = 4,
    PA_ESPI_ALIGN_1024_BYTES = 5,
    PA_ESPI_ALIGN_2048_BYTES = 6,
    PA_ESPI_ALIGN_4096_BYTES = 7
};
#ifndef __cplusplus
typedef enum PromiraEspiAlign PromiraEspiAlign;
#endif

struct PromiraEspiOperatingCfg {
    PromiraSpiIOMode    io_mode;
    PromiraEspiAlertPin alert_pin;
    PromiraEspiAlign    perif_max_req_size;
    PromiraEspiAlign    perif_max_payload;
    u08                 vw_max_count;
    PromiraEspiAlign    oob_max_payload;
    PromiraEspiAlign    flash_max_req_size;
    PromiraEspiAlign    flash_max_payload;
};
#ifndef __cplusplus
typedef struct PromiraEspiOperatingCfg PromiraEspiOperatingCfg;
#endif

/* Set the operating configuration of eSPI system */
int pa_espi_operating_config (
    PromiraConnectionHandle         conn,
    u08                             slave_id,
    const PromiraEspiOperatingCfg * cfg
);


/* Read the operation configuration that eSPI system is using */
int pa_espi_operating_config_read (
    PromiraConnectionHandle   conn,
    u08                       slave_id,
    PromiraEspiOperatingCfg * cfg
);


/* Status - error */
#define PA_READ_ESPI_ERR_TYPE_MASK 0x000000c0
#define PA_READ_ESPI_ERR_TYPE_MASTER 0x00000080
#define PA_READ_ESPI_ERR_TYPE_SLAVE 0x00000040
#define PA_READ_ESPI_ERR_TYPE_MISC 0x000000c0
#define PA_READ_ESPI_ERR_FATAL_MASK 0x00000020
/* Master Error Code - Fatal */
#define PA_READ_ESPI_MST_INVALID_RSP_CODE 0x000000a0
#define PA_READ_ESPI_MST_INVALID_CYCLE_TYPE 0x000000a1
#define PA_READ_ESPI_MST_NO_RSP 0x000000a3
#define PA_READ_ESPI_MST_RSP_FATAL 0x000000a4
#define PA_READ_ESPI_MST_PERIF_PAYLOAD 0x000000a5
#define PA_READ_ESPI_MST_PERIF_REQ_SIZE 0x000000a6
#define PA_READ_ESPI_MST_PERIF_4K_XING 0x000000a7
#define PA_READ_ESPI_MST_VW_MAX_COUNT 0x000000a8
#define PA_READ_ESPI_MST_OOB_PAYLOAD 0x000000a9
#define PA_READ_ESPI_MST_FLASH_PAYLOAD 0x000000aa
#define PA_READ_ESPI_MST_FLASH_REQ_SIZE 0x000000ab
#define PA_READ_ESPI_MST_PERIF_PAYLOAD_SIZE 0x000000ad
#define PA_READ_ESPI_MST_FLASH_PAYLOAD_SIZE 0x000000ae
/* Master Error Code - Non Fatal */
#define PA_READ_ESPI_MST_RSP_NON_FATAL 0x00000080
/* Slave Error Code - Fatal */
#define PA_READ_ESPI_SLV_PUT_WO_FREE 0x00000060
#define PA_READ_ESPI_SLV_GET_WHEN_UNAVAIL 0x00000061
#define PA_READ_ESPI_SLV_PERIF_PAYLOAD 0x0000006a
#define PA_READ_ESPI_SLV_PERIF_REQ_SIZE 0x00000062
#define PA_READ_ESPI_SLV_PERIF_4K_XING 0x00000063
#define PA_READ_ESPI_SLV_VW_MAX_COUNT 0x00000064
#define PA_READ_ESPI_SLV_OOB_PAYLOAD 0x00000065
#define PA_READ_ESPI_SLV_FLASH_PAYLOAD 0x00000066
#define PA_READ_ESPI_SLV_FLASH_REQ_SIZE 0x00000067
#define PA_READ_ESPI_SLV_PERIF_PAYLOAD_SIZE 0x00000068
#define PA_READ_ESPI_SLV_FLASH_PAYLOAD_SIZE 0x00000069
/* Slave Error Code - Non Fatal */
#define PA_READ_ESPI_SLV_INVALID_CMD 0x00000040
#define PA_READ_ESPI_SLV_INVALID_CYCLE_TYPE 0x00000041
/* Miscellaneous Error Code */
#define PA_READ_ESPI_PARTIAL_BYTE 0x000000e0
#define PA_READ_ESPI_RESET_WHILE_CS 0x000000e1
#define PA_READ_ESPI_ALERT_WHILE_CS 0x000000e2
#define PA_READ_ESPI_INVALID_LENGTH 0x000000e3
#define PA_READ_ESPI_MORE_THAN_ONE_CS 0x000000e4
#define PA_READ_ESPI_ERR_BAD_CMD_CRC 0x00000100
#define PA_READ_ESPI_ERR_BAD_RSP_CRC 0x00000200
/* eSPI specific event */
#define PA_EVENT_ESPI_ALERT_RISING 0x00010000
#define PA_EVENT_ESPI_ALERT_FALLING 0x00020000
#define PA_EVENT_ESPI_RESET_RISING 0x00040000
#define PA_EVENT_ESPI_RESET_FALLING 0x00080000
#define PA_EVENT_ESPI_INBAND_RESET 0x00100000
#define PA_EVENT_ESPI_PACKET 0x00200000
/* eSPI transaction base information */
enum PromiraEspiChannel {
    PA_ESPI_CHANNEL_UNKNOWN = -1,
    PA_ESPI_CHANNEL_PERIF   =  0,
    PA_ESPI_CHANNEL_VW      =  1,
    PA_ESPI_CHANNEL_OOB     =  2,
    PA_ESPI_CHANNEL_FLASH   =  3,
    PA_ESPI_CHANNEL_INDEP   =  4
};
#ifndef __cplusplus
typedef enum PromiraEspiChannel PromiraEspiChannel;
#endif

enum PromiraEspiEnumFreq {
    PA_ESPI_ENUM_FREQ_UNKNOWN = -1,
    PA_ESPI_ENUM_FREQ_20MHZ   =  0,
    PA_ESPI_ENUM_FREQ_25MHZ   =  1,
    PA_ESPI_ENUM_FREQ_33MHZ   =  2,
    PA_ESPI_ENUM_FREQ_50MHZ   =  3,
    PA_ESPI_ENUM_FREQ_66MHZ   =  4
};
#ifndef __cplusplus
typedef enum PromiraEspiEnumFreq PromiraEspiEnumFreq;
#endif

/* eSPI packet information */
struct PromiraEspiPacketInfo {
    PromiraEspiChannel  channel;
    PromiraEspiEnumFreq enum_freq;
    PromiraSpiIOMode    io_mode;
    u16                 length;
    u16                 cmd_length;
};
#ifndef __cplusplus
typedef struct PromiraEspiPacketInfo PromiraEspiPacketInfo;
#endif

/* Read eSPI transaction */
int pa_espi_read (
    PromiraConnectionHandle conn,
    PromiraReadInfo *       info,
    PromiraEspiPacketInfo * pkt_info,
    u32                     max_bytes,
    u08 *                   packet
);


struct PromiraEspiStats {
    u32 ch_perif;
    u32 ch_vw;
    u32 ch_oob;
    u32 ch_flash;
    u32 get_cfg;
    u32 set_cfg;
    u32 get_sts;
    u32 pltf_reset;
    u32 alert;
    u32 reset;
    u32 cmd_crc;
    u32 resp_crc;
    u32 fltr_out_pkts;
    u32 fltr_out_cmds;
};
#ifndef __cplusplus
typedef struct PromiraEspiStats PromiraEspiStats;
#endif

/* Get the statistics */
int pa_espi_stats_read (
    PromiraConnectionHandle conn,
    u08                     slave_id,
    PromiraEspiStats *      stats
);


/* For channel match bitmask */
#define PA_ESPI_CHANNEL_MATCH_PERIF 0x00000001
#define PA_ESPI_CHANNEL_MATCH_VW 0x00000002
#define PA_ESPI_CHANNEL_MATCH_OOB 0x00000004
#define PA_ESPI_CHANNEL_MATCH_FLASH 0x00000008
#define PA_ESPI_CHANNEL_MATCH_INDEP 0x00000010
/* eSPI packet matching configuration */
struct PromiraEspiPacketMatch {
    u08              ch_match_bitmask;
    PromiraMatchType cmd_match_type;
    u08              cmd_match_val;
};
#ifndef __cplusplus
typedef struct PromiraEspiPacketMatch PromiraEspiPacketMatch;
#endif

/* Hardware filtering configuration */
int pa_espi_hw_filter_config (
    PromiraConnectionHandle        conn,
    u08                            slave_id,
    const PromiraEspiPacketMatch * pkt_match
);


/* Condtions for trigger */
#define PA_ESPI_ACT_ON_DIG_IN_MASK 0x00000fff
#define PA_ESPI_ACT_ON_ALERT 0x00010000
#define PA_ESPI_ACT_ON_RESET 0x00040000
#define PA_ESPI_ACT_ON_SLAVE_BITMASK 0xf0000000
/* Actions when triggered */
#define PA_ESPI_ACTION_DIG_OUT_MASK 0x00000fff
#define PA_ESPI_ACTION_TRIGGER 0x80000000
/* Simple trigger configuration */
int pa_espi_simple_trigger_config (
    PromiraConnectionHandle        conn,
    u32                            act_on,
    const PromiraEspiPacketMatch * pkt_match,
    u32                            actions
);


/* Array_u08: 16 bytes long */
struct PromiraEspiAdvancedTrigBytes16 {
    u08 byte0;
    u08 byte1;
    u08 byte2;
    u08 byte3;
    u08 byte4;
    u08 byte5;
    u08 byte6;
    u08 byte7;
    u08 byte8;
    u08 byte9;
    u08 byteA;
    u08 byteB;
    u08 byteC;
    u08 byteD;
    u08 byteE;
    u08 byteF;
};
#ifndef __cplusplus
typedef struct PromiraEspiAdvancedTrigBytes16 PromiraEspiAdvancedTrigBytes16;
#endif

/* Array_u08: 2 bytes long */
struct PromiraEspiAdvancedTrigBytes2 {
    u08 byte0;
    u08 byte1;
};
#ifndef __cplusplus
typedef struct PromiraEspiAdvancedTrigBytes2 PromiraEspiAdvancedTrigBytes2;
#endif

/* Advanced Trigger Option 1 */
struct PromiraEspiAdvancedTrig1 {
    u08                              cmd_byte;
    u08                              cmd_cyc;
    u08                              cmd_tag;
    u16                              cmd_len;
    u64                              cmd_addr;
    PromiraEspiAdvancedTrigBytes16   cmd_data;
    PromiraEspiAdvancedTrigBytes16   cmd_data_mask;
    u08                              rsp_byte;
    u08                              rsp_cyc;
    u08                              rsp_tag;
    u16                              rsp_len;
    u64                              rsp_addr;
    PromiraEspiAdvancedTrigBytes16   rsp_data;
    PromiraEspiAdvancedTrigBytes16   rsp_data_mask;
    PromiraEspiAdvancedTrigBytes2    sts_byte;
    PromiraEspiAdvancedTrigBytes2    sts_mask;
    u08                              trg_pin;
    u08                              trg_pin_polarity;
    u08                              trg_pin_direction;
    u08                              cmd_byte_enable;
    u08                              cmd_cyc_enable;
    u08                              cmd_tag_enable;
    u08                              cmd_len_enable;
    u08                              cmd_addr_enable;
    u08                              cmd_data_enable;
    u08                              rsp_byte_enable;
    u08                              rsp_cyc_enable;
    u08                              rsp_tag_enable;
    u08                              rsp_len_enable;
    u08                              rsp_addr_enable;
    u08                              rsp_data_enable;
    u08                              sts_byte_enable;
    u08                              trg_pin_enable;
    u08                              lvl_select_enable;
    u08                              lvl_select_immediate;
};
#ifndef __cplusplus
typedef struct PromiraEspiAdvancedTrig1 PromiraEspiAdvancedTrig1;
#endif

/* Advanced Trigger Option 2 */
struct PromiraEspiAdvancedTrig2 {
    u08                              cmd_byte2;
    u08                              cmd_cyc2;
    u08                              cmd_tag2;
    u16                              cmd_len2;
    u64                              cmd_addr2;
    PromiraEspiAdvancedTrigBytes16   cmd_data2;
    PromiraEspiAdvancedTrigBytes16   cmd_data_mask2;
    u08                              trg_pin_req;
    u08                              trg_pin_req_polarity;
    u08                              trg_pin_req_direction;
    u08                              trg_pin_cmpl0;
    u08                              trg_pin_cmpl0_polarity;
    u08                              trg_pin_cmpl0_direction;
    u08                              trg_pin_cmpl1;
    u08                              trg_pin_cmpl1_polarity;
    u08                              trg_pin_cmpl1_direction;
    u08                              trg_pin_cmpl2;
    u08                              trg_pin_cmpl2_polarity;
    u08                              trg_pin_cmpl2_direction;
    u08                              cmd_byte2_enable;
    u08                              cmd_cyc2_enable;
    u08                              cmd_tag2_enable;
    u08                              cmd_len2_enable;
    u08                              cmd_addr2_enable;
    u08                              cmd_data2_enable;
    u08                              succ_cmpl_enable;
    u08                              unsucc_cmpl_enable;
    u08                              trg_pin_req_enable;
    u08                              trg_pin_cmpl0_enable;
    u08                              trg_pin_cmpl1_enable;
    u08                              trg_pin_cmpl2_enable;
};
#ifndef __cplusplus
typedef struct PromiraEspiAdvancedTrig2 PromiraEspiAdvancedTrig2;
#endif

/* Advanced Trigger Option Error */
struct PromiraEspiAdvancedTrigError {
    u08 err_code;
    u08 err_code_enable;
};
#ifndef __cplusplus
typedef struct PromiraEspiAdvancedTrigError PromiraEspiAdvancedTrigError;
#endif

/* Advanced Trigger Configuration */
int pa_espi_adv_trig_config (
    PromiraConnectionHandle              conn,
    u08                                  slave_id,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level1,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level2,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level3,
    const PromiraEspiAdvancedTrig1 *     pkt_adv_trig1_level4,
    const PromiraEspiAdvancedTrig2 *     pkt_adv_trig2,
    const PromiraEspiAdvancedTrigError * pkt_adv_trig_error
);




#ifdef __cplusplus
}
#endif

#endif  /* __promana_h__ */
